function theData = XPCS_fileReader(theFiles, varargin)
% theData = XPCS_fileReader(theFiles, typeOption, returnOption): 
% Reads .hdf and .fits file(s) and returns the data
%
% theData - output of series of images, a single image, or single value
%  depending on the return option
% theFiles - list of files to be opened (with path and extension)
% typeOption - double to specify file type
%   0: check each input file for data type
%   1: all input files are .hdf
%   2: all input files are .fits
% returnOption - string to specify output type
%   all: simply returns all opened files in 3D array
%   mean: average all of the input files and return a single 2D image
%   min: average all of the input files and return the minimum as a double
%   first: read in just the first file and return the first image
% Options may be given in either order or omitted. If typeOption is
% omitted, 0 is used. If returnOption is omitted, 'all' is used.
%
% Copyright 2010
% Sebastian Dietze
% University of California, San Diego

% Revision: 1.0  Date: 2010/10/24

option1 = 0;
option2 = 'all';
for k = 1:nargin-1
   if(isa(varargin{k}, 'numeric'))
     option1 = varargin{k};
     dataType = option1;
   elseif(isa(varargin{k}, 'char'))
     option2 = varargin{k};
   end
end

if(isa(theFiles, 'char'))
  %convert to a single cell to keep format same
  theFiles = cellstr(theFiles); %May be irrelavent
end

switch option2
  case 'mean'
    theData = 0;
  case 'min'
    theData = 0;
  otherwise
    %do nothing  
end

numFil = size(theFiles, 1);
totSize = 1;
for k = 1:numFil
  if(option1 == 0)
    %find out data type
    [pathstr, name, ext] = fileparts(theFiles{k});
    switch ext
      case '.hdf'
        dataType = 1;
      case '.fits'
        dataType = 2;
    end
  end
  switch dataType
    case 1
      % .hdf files
      switch option2
        case 'mean'
          %return mean of all
          theData = theData + double(hdfread(theFiles{k},'data'));
        case 'min'
          %take mean of all and return min
          theData = theData + double(hdfread(theFiles{k},'data'));
        otherwise
          %just return all
          theData(:,:,totSize) = hdfread(theFiles{k},'data');
      end
      totSize = totSize + 1;
    case 2
      % .fits files
      infoFits = fitsinfo(theFiles{k});
      %Check the Dimension of Primary Array and Read in non-empty Data
      if(isempty(infoFits.PrimaryData.Size))
        %Read Image FITS data if single image
        switch option2
          case 'mean'
            %return mean of all
            theData = theData + double(fitsread(theFiles{k},'image'));
          case 'min'
            %take mean of all and return min
            theData = theData + double(fitsread(theFiles{k},'image'));
          otherwise
            %just return all
            theData(:,:,totSize) = fitsread(theFiles{k},'image');
        end
        totSize = totSize + 1;
      else
        %Read Primary FITS Data if multiple images
        switch option2
          case 'mean'
            %return mean of all
            theData = theData + sum(double(fitsread(theFiles{k}),3));
          case 'min'
            %take mean of all and return min
            theData = theData + sum(double(fitsread(theFiles{k}),3));
          otherwise
            %just return all
            theData(:,:,totSize:totSize+infoFits.PrimaryData.Size(3)-1) ...
                = fitsread(theFiles{k});
        end
        totSize = totSize + infoFits.PrimaryData.Size(3);
      end
  end
end


switch option2
    case 'mean'
        %return mean of all
        theData = theData/totSize;
    case 'min'
        theData = min(min(theData))/totSize;
    case 'first'
        %take first one only
        theData = theData(:,:,1);
    otherwise
        %do nothing and return all
end

%-----
% EOF
%-----