function [factor imgOut] = XPCS_imageSub(img1, img2, varargin)
% XPCS_imageSub: Takes the difference of two images to take out background

% imgOut = XPCS(img1, img2): imgOut = img1 - img2
%  imgOut is an output array the same size as img1 and img2,
%  while img1 and img2 can be either input
%  arrays or .hdf etc. path\file references

% imgOut = XPCS(img1, img2, factor): imgOut = img1 - factor*img2

% imgOut = XPCS(img1, img2, option): computes a factor automatically
% between the low and high limits based on the following option:
% 'sum' - attempts to minimize sum(sum(imgOut));
% 'min' - scales to min value in image2;
% 'max' - scales to max value in image2;
% 'bot' - similar to sum for values lower than 1.25*min(min(image2));
% 'top' - similar to sum for values higher than .75*max(max(image2));
% 'flo' - makes min(min(imgOut)) == 0;

% Copyright 2010
% Sebastian Dietze
% University of California, San Diego

% Revision: 1.0  Date: 2010/08/28

%check that input 1 and 2 are strings or arrays

if(isa(img1,'char'))
	image1 =  double(hdfread(img1,'data'));
elseif(isa(img1,'numeric'))
	image1 = img1;
else
    error('myApp:argChk', 'img1 is neither a string nor a number array')
end
if(isa(img2,'char'))
	image2 =  double(hdfread(img2,'data'));
elseif(isa(img2,'numeric'))
	image2 = img2;
else
    error('myApp:argChk', 'img2 is neither a string nor a number array')
end

optargin = size(varargin,2);

switch optargin
  case 0
    imgOut = image1 - image2;
    factor = 1;
  case 1
    if (isa(varargin{1,1}, 'numeric'))
      factor = varargin{1,1};
      imgOut = image1 - factor*image2;
    elseif (isa(varargin{1,1}, 'char'))
      option = varargin{1,1};
      switch option
        case 'Sum'
          tmp = [min(min(image1))/min(min(image2)) ...
              max(max(image1))/max(max(image2))];
          theLow = min(tmp)-.1*min(tmp);
          theHigh = max(tmp)+.1*max(tmp);
          leftMid = (3*theLow+theHigh)/4;
          rightMid = (theLow+3*theHigh)/4;
          lefttSum = abs(sum(sum(image1 - leftMid*image2)));
          rightSum = abs(sum(sum(image1 - rightMid*image2)));
          while(rightMid - leftMid > .1 || abs(lefttSum - rightSum)/(lefttSum + rightSum) > .01)
            if(rightSum < lefttSum)
              theLow = leftMid;
            else
              theHigh = rightMid;
            end
            leftMid = (3*theLow+theHigh)/4;
            rightMid = (theLow+3*theHigh)/4;
            lefttSum = abs(sum(sum(image1 - leftMid*image2)));
            rightSum = abs(sum(sum(image1 - rightMid*image2)));
          end
          factor = (theLow+theHigh)/2;
          imgOut = image1 - factor*image2;
        case 'MinM'
          theMin = min(min(image1));
          factor = theMin/image2(image1 == theMin);
          imgOut = image1 - factor*image2;
        case 'MaxM'
          theMax = max(max(image1));
          factor = theMax/image2(image1 == theMax);
          imgOut = image1 - factor*image2;
        case 'MinS'
          theMin = min(min(image2));
          factor = image1(image2 == theMin)/theMin;
          imgOut = image1 - factor*image2;
        case 'MaxS'
          %theMax = max(max(image2));
          %factor = image1(image2 == theMax)/theMax;
          
          theMax = max(max(image2));
          factor = mean(image1(image2 >= .7*theMax & image2 <= .8*theMax))/theMax;
          imgOut = image1 - factor*image2;
        case 'Bottom'
          reg = min(min(image2))+.25*abs(min(min(image2)));
          tmp = [min(min(image1(image2<reg)))/min(min(image2(image2<reg))) ...
              max(max(image1(image2<reg)))/max(max(image2(image2<reg)))];
          theLow = min(tmp)-.1*min(tmp);
          theHigh = max(tmp)+.1*max(tmp);
          leftMid = (3*theLow+theHigh)/4;
          rightMid = (theLow+3*theHigh)/4;
          lefttSum = abs(sum(image1(image2<reg) - leftMid*image2(image2<reg)));
          rightSum = abs(sum(image1(image2<reg) - rightMid*image2(image2<reg)));
          while(rightMid - leftMid > .1 || abs(lefttSum - rightSum)/(lefttSum + rightSum) > .01)
            if(rightSum < lefttSum)
              theLow = leftMid;
            else
              theHigh = rightMid;
            end
            leftMid = (3*theLow+theHigh)/4;
            rightMid = (theLow+3*theHigh)/4;
            lefttSum = abs(sum(image1(image2<reg) - leftMid*image2(image2<reg)));
            rightSum = abs(sum(image1(image2<reg) - rightMid*image2(image2<reg)));
          end
          factor = (theLow+theHigh)/2;
          imgOut = image1 - factor*image2;
        case 'Top'
          reg = max(max(image2))-.25*abs(max(max(image2)));
          tmp = [min(min(image1(image2>reg)))/min(min(image2(image2>reg))) ...
              max(max(image1(image2>reg)))/max(max(image2(image2>reg)))];
          theLow = min(tmp)-.1*min(tmp);
          theHigh = max(tmp)+.1*max(tmp);
          leftMid = (3*theLow+theHigh)/4;
          rightMid = (theLow+3*theHigh)/4;
          lefttSum = abs(sum(image1(image2>reg) - leftMid*image2(image2>reg)));
          rightSum = abs(sum(image1(image2>reg) - rightMid*image2(image2>reg)));
          while(rightMid - leftMid > .1 || abs(lefttSum - rightSum)/(lefttSum + rightSum) > .01)
            if(rightSum < lefttSum)
              theLow = leftMid;
            else
              theHigh = rightMid;
            end
            leftMid = (3*theLow+theHigh)/4;
            rightMid = (theLow+3*theHigh)/4;
            lefttSum = abs(sum(image1(image2>reg) - leftMid*image2(image2>reg)));
            rightSum = abs(sum(image1(image2>reg) - rightMid*image2(image2>reg)));
          end
          factor = (theLow+theHigh)/2;
          imgOut = image1 - factor*image2;
        case 'Floor'
          factor = 1;
          for k = 1:3
            diff = image1 - factor*image2;
            del = min(min(diff));
            factor = image1(diff == del)/image2(diff == del);
          end
          imgOut = image1 - factor*image2;
          
      end
    end
end



%-----
%-EOF-
%-----